@@ -1,12 +1,11 @@
 """Classes related to doses and plans."""
 
-import numpy as np
-import os
 import numbers
-import pydicom
 import functools
 
 import matplotlib
+import numpy as np
+import pydicom
 
 import skrt.core
 import skrt.image
@@ -15,13 +14,13 @@ import skrt.structures
 
 class ImageOverlay(skrt.image.Image):
     """Image that can be associated with another Image, and whose plotting
-    functionality includes the ability to plot overlaid on its associated 
+    functionality includes the ability to plot overlaid on its associated
     Image."""
 
-    def __init__(self, path="", load=True, image=None, *args, **kwargs):
-
-        kwargs['default_intensity'] = kwargs.get('default_intensity', None)
-        skrt.image.Image.__init__(self, path, load, *args, **kwargs)
+    def __init__(self, path="", load=True, image=None, **kwargs):
+        kwargs["default_intensity"] = kwargs.get("default_intensity", None)
+        super().__init__(path=path, load=load, **kwargs)
+        self.image = None
         self.set_image(image)
 
         # Default dose plotting settings
@@ -41,7 +40,7 @@ class ImageOverlay(skrt.image.Image):
         return self.image
 
     def plot(
-        self, 
+        self,
         view=None,
         sl=None,
         idx=None,
@@ -51,14 +50,18 @@ class ImageOverlay(skrt.image.Image):
         figsize=None,
         zoom=None,
         colorbar=False,
+        xlabel=None,
+        ylabel=None,
         no_xlabel=False,
         no_ylabel=False,
+        xticks=None,
+        yticks=None,
         no_xticks=False,
         no_yticks=False,
         no_xtick_labels=False,
         no_ytick_labels=False,
-        include_image=True, 
-        opacity=None, 
+        include_image=True,
+        opacity=None,
         intensity=None,
         mpl_kwargs=None,
         show=True,
@@ -67,7 +70,7 @@ class ImageOverlay(skrt.image.Image):
         masked=True,
         invert_mask=False,
         mask_color="black",
-        **kwargs
+        **kwargs,
     ):
         """Plot this overlay, optionally overlaid on its associated Image.
 
@@ -104,11 +107,11 @@ class ImageOverlay(skrt.image.Image):
 
 
         include_image : bool, default=False
-            If True and this ImageOverlay has an associate image, it will be 
+            If True and this ImageOverlay has an associate image, it will be
             plotted overlaid on the image.
-        
+
         opacity : float, default=None
-            If plotting on top of an image, this sets the opacity of the 
+            If plotting on top of an image, this sets the opacity of the
             overlay (0 = fully transparent, 1 = fully opaque).
 
         mpl_kwargs : dict, default=None
@@ -121,12 +124,28 @@ class ImageOverlay(skrt.image.Image):
             or overlay;
             - 0 or False: no colour bar.
 
+        xlabel : str, default=None
+            Custom label for the x axis.  If None, the label is set to
+            be the coordinate axis and units, for example "x (mm)".
+            If False or '', the x axis will not be labelled.
+
+        ylabel : str, default=None
+            Custom label for the y axis.  If None, the label is set to
+            be the coordinate axis and units, for example "y (mm)".
+            If False or '', the x axis will not be labelled.
+
         no_xlabel : bool, default=False
             If True, the x axis will not be labelled.
 
         no_ylabel : bool, default=False
             If True, the y axis will not be labelled.
 
+        xticks : tuple/dict, default=None
+            Tick locations and custom tick labels for the x axis.
+
+        yticks : tuple/dict, default=None
+            Tick locations and custom tick labels for the y axis.
+
         no_xticks : bool, default=False
             If True, ticks (and their labels) on the x axis will not be shown.
 
@@ -175,15 +194,31 @@ class ImageOverlay(skrt.image.Image):
 
         # Plot underlying image, always without title.
         if include_image and self.image is not None:
-            self.image.plot(view, sl=sl, idx=idx, pos=pos, ax=self.ax,
-                    show=False, title="", colorbar=max((colorbar - 1), 0),
-                    no_xlabel=no_xlabel, no_ylabel=no_ylabel,
-                    no_xticks=no_xticks, no_yticks=no_yticks,
-                    no_xtick_labels=no_xtick_labels,
-                    no_ytick_labels=no_ytick_labels,
-                    mask=mask, mask_threshold=mask_threshold,
-                    masked=masked, invert_mask=invert_mask,
-                    mask_color=mask_color)
+            self.image.plot(
+                view,
+                sl=sl,
+                idx=idx,
+                pos=pos,
+                ax=self.ax,
+                show=False,
+                title="",
+                colorbar=max((colorbar - 1), 0),
+                xlabel=xlabel,
+                ylabel=ylabel,
+                no_xlabel=no_xlabel,
+                no_ylabel=no_ylabel,
+                xticks=xticks,
+                yticks=yticks,
+                no_xticks=no_xticks,
+                no_yticks=no_yticks,
+                no_xtick_labels=no_xtick_labels,
+                no_ytick_labels=no_ytick_labels,
+                mask=mask,
+                mask_threshold=mask_threshold,
+                masked=masked,
+                invert_mask=invert_mask,
+                mask_color=mask_color,
+            )
 
             # Use default transprency if plotting image and opacity is None
             if opacity is None:
@@ -204,7 +239,7 @@ class ImageOverlay(skrt.image.Image):
 
         # Plot self
         skrt.image.Image.plot(
-            self, 
+            self,
             view=view,
             sl=sl,
             idx=idx,
@@ -212,10 +247,14 @@ class ImageOverlay(skrt.image.Image):
             ax=self.ax,
             zoom=zoom,
             colorbar=colorbar,
-            mpl_kwargs=mpl_kwargs, 
-            show=show, 
+            mpl_kwargs=mpl_kwargs,
+            show=show,
+            xlabel=xlabel,
+            ylabel=ylabel,
             no_xlabel=no_xlabel,
             no_ylabel=no_ylabel,
+            xticks=xticks,
+            yticks=yticks,
             no_xticks=no_xticks,
             no_yticks=no_yticks,
             no_xtick_labels=no_xtick_labels,
@@ -225,15 +264,15 @@ class ImageOverlay(skrt.image.Image):
             masked=masked,
             invert_mask=invert_mask,
             mask_color=mask_color,
-            **kwargs
+            **kwargs,
         )
 
     @functools.cached_property
     def max(self):
+        """Return maximum data value."""
         return self.get_data().max()
 
 
-
 class Dose(ImageOverlay):
     """
     Class representing a radiotherapy dose field.  This is the
@@ -246,8 +285,7 @@ class Dose(ImageOverlay):
     """
 
     def __init__(self, *args, **kwargs):
-
-        ImageOverlay.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         # Ensure linking between image and dose
         self.set_image(self.image)
@@ -258,7 +296,7 @@ class Dose(ImageOverlay):
         self._default_opacity = 0.5
 
         # Delete spurious attributes inherited from Image class
-        for attribute in ['doses', 'plans']:
+        for attribute in ["doses", "plans"]:
             if hasattr(self, attribute):
                 delattr(self, attribute)
 
@@ -268,6 +306,7 @@ class Dose(ImageOverlay):
         self.dose_summation_type = None
         self.n_fraction = None
         self._equivalent_dose = None
+        self.plan = None
 
     def load(self, *args, **kwargs):
         """Load self and set default maximum plotting intensity from max of
@@ -275,11 +314,11 @@ class Dose(ImageOverlay):
 
         skrt.image.Image.load(self, *args, **kwargs)
         self._default_vmax = self.data.max()
-        ds = self.dicom_dataset
-        if ds:
-            self.dose_units = getattr(ds, 'DoseUnits', None)
-            self.dose_type = getattr(ds, 'DoseType', None)
-            self.dose_summation_type = getattr(ds, 'DoseSummationType', None)
+        dset = self.dicom_dataset
+        if dset:
+            self.dose_units = getattr(dset, "DoseUnits", None)
+            self.dose_type = getattr(dset, "DoseType", None)
+            self.dose_summation_type = getattr(dset, "DoseSummationType", None)
 
     def copy_dicom(self, *args, **kwargs):
         """
@@ -315,14 +354,53 @@ class Dose(ImageOverlay):
             self.plan.add_dose(self)
 
     def get_dose_units(self):
+        """
+        Get unit of dose.
+
+        Possible values include:
+
+        - "GY": Gray;
+        - "RELATIVE": dose relative to a reference dose.
+
+        For details, see:
+
+        - https://dicom.innolitics.com/ciods/rt-dose/rt-dose/30040002
+        """
         self.load()
         return self.dose_units
 
     def get_dose_type(self):
+        """
+        Get dose type.
+
+        Possible values are:
+
+        - "PHYSICAL": physical dose;
+        - "EFFECTIVE": physical dose adjusted for a given biological effect;
+        - "ERROR": difference between ideal and planned dose.
+
+        For details, see:
+
+        - https://dicom.innolitics.com/ciods/rt-dose/rt-dose/30040004
+        """
         self.load()
         return self.dose_type
 
     def get_dose_summation_type(self):
+        """
+        Get type of dose summation.
+
+        Possible values include:
+
+        - "PLAN": dose summed over all fraction groups;
+        - "MULTI_PLAN": dose summed over two or more plans;
+        - "FRACTION": dose for a single fraction group;
+        - "BEAM": dose for one or more beams.
+
+        For details, see:
+
+        - https://dicom.innolitics.com/ciods/rt-dose/rt-dose/3004000a
+        """
         self.load()
         return self.dose_summation_type
 
@@ -342,44 +420,57 @@ class Dose(ImageOverlay):
             roi = roi.clone()
             roi.set_image(self)
 
-        dose_in_roi = self.get_data(standardise=standardise) \
-                * roi.get_mask(standardise=standardise)
+        dose_in_roi = self.get_data(standardise=standardise) * roi.get_mask(
+            standardise=standardise
+        )
         return dose_in_roi
 
     def get_dose_in_roi(self, roi, standardise=True):
-        """Return 1D numpy array containing all of the dose values for the 
-        voxels inside an ROI. Fails if the ROI and dose arrays are not the 
+        """Return 1D numpy array containing all of the dose values for the
+        voxels inside an ROI. Fails if the ROI and dose arrays are not the
         same size."""
         dose_in_roi = self.get_dose_in_roi_3d(roi, standardise)
         return dose_in_roi[dose_in_roi > 0]
 
-    def get_max_dose_in_rois(self, rois=[]):
-        '''
+    def get_max_dose_in_rois(self, rois=None):
+        """
         Return maximum dose in a set of rois.
 
         **Parameter:**
 
-        rois : list, default=[]
+        rois : list, default=None
             List of ROI objects, for which maximum dose is to be determined.
-        '''
+        """
         # Determine the maximum dose for the input roi(s).
         dose_max = 0
-        for roi in rois:
-            doses = list(self.get_dose_in_roi(roi))
-            doses.append(dose_max)
-            dose_max=max(doses)
+        if rois:
+            for roi in rois:
+                doses = list(self.get_dose_in_roi(roi))
+                doses.append(dose_max)
+                dose_max = max(doses)
         return dose_max
 
-    def plot_dvh(self, rois=[], bins=50, dose_min=0, dose_max=None,
-            figsize=(8, 4), lw=2, n_colour=None, cmap='turbo', grid=True,
-            fname=None, legend_bbox_to_anchor=(1.01, 0.5),
-            legend_loc='center left'):
-        '''
+    def plot_dvh(
+        self,
+        rois=None,
+        bins=50,
+        dose_min=0,
+        dose_max=None,
+        figsize=(8, 4),
+        lw=2,
+        n_colour=None,
+        cmap="turbo",
+        grid=True,
+        fname=None,
+        legend_bbox_to_anchor=(1.01, 0.5),
+        legend_loc="center left",
+    ):
+        """
         Plot dose-volume histogram for specified ROI(s).
 
         **Parameters:**
 
-        rois : ROI/StructureSet/list, default=[]
+        rois : ROI/StructureSet/list, default=None
             ROI(s) for which dose-volume histogram is to be plotted.  This
             can be a single skrt.structures.ROI object, a single
             skrt.structures.StructureSet object, or a list containing any
@@ -420,7 +511,7 @@ class Dose(ImageOverlay):
 
         fname : str, default=None
             Name of file for saving output.
-        '''
+        """
 
         # Obtain a list containing all unique rois.
         all_rois = skrt.structures.get_all_rois(rois)
@@ -430,31 +521,41 @@ class Dose(ImageOverlay):
             dose_max = self.get_max_dose_in_rois(all_rois)
 
         # Create figure and define the list of colours.
-        fig, ax = matplotlib.pyplot.subplots(figsize=figsize)
+        axes = matplotlib.pyplot.subplots(figsize=figsize)[1]
         if n_colour is None:
             colours = [roi.color for roi in all_rois]
         else:
             colours = matplotlib.cm.get_cmap(cmap)(np.linspace(0, 1, n_colour))
-        ax.set_prop_cycle(color=colours)
+        axes.set_prop_cycle(color=colours)
 
         # Plot the dose-volume histograms, and extract information for legend.
-        lines= []
-        labels= []
+        lines = []
+        labels = []
         for roi in all_rois:
             doses = self.get_dose_in_roi(roi)
-            n, bins, patches = ax.hist(doses, bins=bins,
-                    range=(dose_min,dose_max), lw=lw, histtype='step',
-                    density=True, cumulative=-1)
+            bins, patches = axes.hist(
+                doses,
+                bins=bins,
+                range=(dose_min, dose_max),
+                lw=lw,
+                histtype="step",
+                density=True,
+                cumulative=-1,
+            )[1:]
             colour = patches[0].get_edgecolor()
             lines.append(matplotlib.lines.Line2D([0], [0], color=colour, lw=lw))
             labels.append(roi.name)
 
         # Label the axes, add grid and legend, tighten layout.
-        ax.set_xlabel('Dose (Gy)')
-        ax.set_ylabel('Volume fraction')
-        ax.grid(grid)
-        ax.legend(handles=lines, labels=labels,
-                bbox_to_anchor=legend_bbox_to_anchor, loc=legend_loc)
+        axes.set_xlabel("Dose (Gy)")
+        axes.set_ylabel("Volume fraction")
+        axes.grid(grid)
+        axes.legend(
+            handles=lines,
+            labels=labels,
+            bbox_to_anchor=legend_bbox_to_anchor,
+            loc=legend_loc,
+        )
         matplotlib.pyplot.tight_layout()
 
         # Show figure or save to file.
@@ -463,7 +564,7 @@ class Dose(ImageOverlay):
         else:
             matplotlib.pyplot.show()
 
-        return ax
+        return axes
 
     def plot_DVH(self, *args, **kwargs):
         """Alias for plot_dvh()."""
@@ -475,8 +576,9 @@ class Dose(ImageOverlay):
         doses = self.get_dose_in_roi(roi)
         if doses.size:
             return np.mean(doses)
-        elif roi.get_volume(method="mask"):
+        if roi.get_volume(method="mask"):
             return 0
+        return None
 
     def get_dose_quantile(self, roi, quantile=0.5):
         """Get specified dose quantile inside an ROI."""
@@ -484,6 +586,7 @@ class Dose(ImageOverlay):
         doses = self.get_dose_in_roi(roi)
         if doses.size:
             return np.quantile(doses, quantile)
+        return None
 
     def get_bed(self, *args, **kwargs):
         """
@@ -495,8 +598,14 @@ class Dose(ImageOverlay):
         return self.get_biologically_effective_dose(*args, **kwargs)
 
     def get_biologically_effective_dose(
-            self, rois=None, alpha_beta_ratios=None,
-            n_fraction=None, fill=0, standardise=False, force=False):
+        self,
+        rois=None,
+        alpha_beta_ratios=None,
+        n_fraction=None,
+        fill=0,
+        standardise=False,
+        force=False,
+    ):
         """
         Get a Dose object where array values are biologically effective dose.
 
@@ -549,9 +658,14 @@ class Dose(ImageOverlay):
         """
         self._equivalent_dose = "get_biologically_effective_dose()"
         return self.get_equivalent_dose(
-                dose_rate=0, rois=rois, alpha_beta_ratios=alpha_beta_ratios,
-                n_fraction=n_fraction, fill=fill, standardise=standardise,
-                force=force)
+            dose_rate=0,
+            rois=rois,
+            alpha_beta_ratios=alpha_beta_ratios,
+            n_fraction=n_fraction,
+            fill=fill,
+            standardise=standardise,
+            force=force,
+        )
 
     def get_eqd(self, *args, **kwargs):
         """
@@ -563,8 +677,15 @@ class Dose(ImageOverlay):
         return self.get_equivalent_dose(*args, **kwargs)
 
     def get_equivalent_dose(
-            self, dose_rate=2, rois=None, alpha_beta_ratios=None,
-            n_fraction=None, fill=0, standardise=False, force=False):
+        self,
+        dose_rate=2,
+        rois=None,
+        alpha_beta_ratios=None,
+        n_fraction=None,
+        fill=0,
+        standardise=False,
+        force=False,
+    ):
         """
         Get a Dose object where array values are equivalent dose
         for a specified dose per fraction.
@@ -628,65 +749,79 @@ class Dose(ImageOverlay):
 
         # Check input values.
         if not isinstance(fill, numbers.Number) and fill is not None:
-            self.logger.error(f"Value of fill set to '{fill}' "
-                    "but must be a number or None.")
+            self.logger.error(
+                "Value of fill set to '%s' but must be a number or None.", fill
+            )
             self._equivalent_dose = None
-            return
+            return None
 
         self.load()
         if "EFFECTIVE" == self.dose_type and not force:
-            self.logger.error(f"Method {self._equivalent_dose} called "
-                    f"for dose with type '{self.dose_type}'.  "
-                    "If this is the intention, repeat call with 'force=True'")
+            self.logger.error(
+                "Method '%s' called for dose with type '%s'.  "
+                "If this is the intention, repeat call with 'force=True'",
+                self._equivalent_dose,
+                self.dose_type,
+            )
             self._equivalent_dose = None
-            return
+            return None
 
         rois = skrt.structures.get_all_rois(rois)
         if not rois:
-            self.logger.error(f"Method {self._equivalent_dose} called "
-                "without specifying any ROIs.")
+            self.logger.error(
+                "Method %s called without specifying any ROIs.",
+                self._equivalent_dose,
+            )
             self._equivalent_dose = None
-            return
+            return None
 
         if not n_fraction:
             n_fraction = self.get_n_fraction()
 
         if not n_fraction:
-            self.logger.error(f"Method {self._equivalent_dose} called "
-                "without specifying number of fractions.")
+            self.logger.error(
+                "Method %s called without specifying number of fractions.",
+                self._equivalent_dose,
+            )
             self._equivalent_dose = None
-            return
+            return None
 
         # Set default dose to be fill value.
         if isinstance(fill, numbers.Number):
             dose = Dose(
-                    path=(fill *
-                        np.ones(self.get_data(standardise=standardise).shape)),
-                    affine=self.get_affine(standardise=standardise))
+                path=(
+                    fill * np.ones(self.get_data(standardise=standardise).shape)
+                ),
+                affine=self.get_affine(standardise=standardise),
+            )
         # Set default dose to be initial dose.
         elif fill is None:
             dose = Dose(
-                    path=self.get_data(standardise=standardise),
-                    affine=self.get_affine(standardise=standardise))
+                path=self.get_data(standardise=standardise),
+                affine=self.get_affine(standardise=standardise),
+            )
 
         # Calculate equivalent dose inside ROIs with alpha/beta defined.
         for roi in rois:
             # For current ROI, use alpha/beta given in input,
             # or use the value assigned to the ROI.
             alpha_over_beta = alpha_beta_ratios.get(
-                    roi.name, roi.alpha_over_beta)
+                roi.name, roi.alpha_over_beta
+            )
             # If alpha/beta is defined,
             # calculate the equivalent dose.
             if alpha_over_beta is not None:
                 dose_in_roi = self.get_dose_in_roi_3d(
-                        roi, standardise=standardise)
-                dose_in_roi *= ((alpha_over_beta + dose_in_roi / n_fraction)
-                                / (dose_rate + alpha_over_beta))
+                    roi, standardise=standardise
+                )
+                dose_in_roi *= (alpha_over_beta + dose_in_roi / n_fraction) / (
+                    dose_rate + alpha_over_beta
+                )
                 dose.data[dose_in_roi > 0] = dose_in_roi[dose_in_roi > 0]
 
         # Reset the default maximum plotting intensity
         # from the maximum of the data array.
-        dose._default_vmax = dose.data.max()
+        setattr(dose, "_default_vmax", dose.data.max())
 
         # Set dose_type to indicate that biological effect
         # is taken into account.
@@ -696,6 +831,7 @@ class Dose(ImageOverlay):
         self._equivalent_dose = None
         return dose
 
+
 class Plan(skrt.core.Archive):
     """
     Class representing a radiotherapy plan.
@@ -730,6 +866,8 @@ class Plan(skrt.core.Archive):
         self.n_beam_seq = None
         self.n_fraction = None
         self.organs_at_risk = None
+        self.machine = None
+        self.station_name = None
         self.target_dose = None
         self.targets = None
         self.image = None
@@ -739,7 +877,7 @@ class Plan(skrt.core.Archive):
         for constraint_attribute in Constraint.get_weight_and_objectives():
             setattr(self.objectives, constraint_attribute, None)
 
-        skrt.core.Archive.__init__(self, path)
+        super().__init__(path)
 
         self.constraints_loaded = False
         if load:
@@ -760,11 +898,14 @@ class Plan(skrt.core.Archive):
 
         self.dicom_dataset = pydicom.dcmread(self.path, force=True)
 
-        self.name = getattr(self.dicom_dataset, 'RTPlanName', None)
+        self.name = getattr(self.dicom_dataset, "RTPlanName", None)
         self.description = getattr(
-                self.dicom_dataset, 'RTPlanDescription', None)
+            self.dicom_dataset, "RTPlanDescription", None
+        )
         self.prescription_description = getattr(
-                self.dicom_dataset, 'PrescriptionDescription', None)
+            self.dicom_dataset, "PrescriptionDescription", None
+        )
+        self.station_name = getattr(self.dicom_dataset, "StationName", None)
 
         try:
             self.approval_status = self.dicom_dataset.ApprovalStatus
@@ -773,7 +914,8 @@ class Plan(skrt.core.Archive):
 
         try:
             self.n_fraction_group = len(
-                    self.dicom_dataset.FractionGroupSequence)
+                self.dicom_dataset.FractionGroupSequence
+            )
         except AttributeError:
             self.n_fraction_group = None
 
@@ -794,13 +936,16 @@ class Plan(skrt.core.Archive):
                                 self.target_dose = 0.0
                             self.target_dose += dose.TargetPrescriptionDose
         dose_reference_sequence = getattr(
-                self.dicom_dataset, 'DoseReferenceSequence', [])
+            self.dicom_dataset, "DoseReferenceSequence", []
+        )
 
         if self.target_dose is None:
             target_dose = 0.0
             for dose_reference in dose_reference_sequence:
-                target_dose = max(target_dose,
-                        getattr(dose_reference, 'TargetPrescriptionDose', 0))
+                target_dose = max(
+                    target_dose,
+                    getattr(dose_reference, "TargetPrescriptionDose", 0),
+                )
             if target_dose:
                 self.target_dose = target_dose
 
@@ -816,7 +961,7 @@ class Plan(skrt.core.Archive):
             If True, dose constraints will be reloaded from file, even if
             loaded previously.
         """
-        if ((self.constraints_loaded and not force) or not self.structure_set):
+        if (self.constraints_loaded and not force) or not self.structure_set:
             return
 
         rois = {}
@@ -825,36 +970,44 @@ class Plan(skrt.core.Archive):
 
         self.organs_at_risk = []
         self.targets = []
-        
-        ds = self.get_dicom_dataset()
 
-        dose_reference_sequence = getattr(ds, 'DoseReferenceSequence', [])
+        dset = self.get_dicom_dataset()
+
+        dose_reference_sequence = getattr(dset, "DoseReferenceSequence", [])
 
         for item in dose_reference_sequence:
-            roi = rois.get(getattr(item, 'ReferencedROINumber', None), None)
+            roi = rois.get(getattr(item, "ReferencedROINumber", None), None)
             if roi is None:
                 continue
             roi.roi_type = item.DoseReferenceType
             roi.constraint = Constraint()
             roi.constraint.weight = skrt.core.get_float(
-                    item, "ConstraintWeight")
-            if 'ORGAN_AT_RISK' == item.DoseReferenceType:
+                item, "ConstraintWeight"
+            )
+            if item.DoseReferenceType in ["ORGAN_AT_RISK", "OAR"]:
                 roi.constraint.maximum_dose = skrt.core.get_float(
-                        item, "OrganAtRiskMaximumDose")
+                    item, "OrganAtRiskMaximumDose"
+                )
                 roi.constraint.full_volume_dose = skrt.core.get_float(
-                        item, "OrganAtRiskFullVolumeDose")
+                    item, "OrganAtRiskFullVolumeDose"
+                )
                 roi.constraint.overdose_volume_fraction = skrt.core.get_float(
-                        item, "OrganAtRiskOverdoseVolumeFraction")
+                    item, "OrganAtRiskOverdoseVolumeFraction"
+                )
                 self.organs_at_risk.append(roi)
-            elif 'TARGET' == item.DoseReferenceType:
+            elif item.DoseReferenceType in ["TARGET"]:
                 roi.constraint.minimum_dose = skrt.core.get_float(
-                        item, "TargetMinimumDose")
+                    item, "TargetMinimumDose"
+                )
                 roi.constraint.prescription_dose = skrt.core.get_float(
-                        item, "TargetPrescriptionDose")
+                    item, "TargetPrescriptionDose"
+                )
                 roi.constraint.maximum_dose = skrt.core.get_float(
-                        item, "TargetMaximumDose")
+                    item, "TargetMaximumDose"
+                )
                 roi.constraint.underdose_volume_fraction = skrt.core.get_float(
-                        item, "TargetUnderdoseVolumeFraction")
+                    item, "TargetUnderdoseVolumeFraction"
+                )
                 self.targets.append(roi)
 
         self.constraints_loaded = True
@@ -880,8 +1033,9 @@ class Plan(skrt.core.Archive):
         self.load()
         return self.organs_at_risk
 
-    def get_dose_objective(self, objective='maximum_dose', idx_dose=0,
-            dose=None):
+    def get_dose_objective(
+        self, objective="maximum_dose", idx_dose=0, dose=None
+    ):
         """
         Obtain Dose object representing weight or objective of dose constraint.
 
@@ -900,8 +1054,8 @@ class Plan(skrt.core.Archive):
         """
         # Check that specified objective is known.
         if objective not in Constraint.get_weight_and_objectives():
-            print(f'Unknown dose objective: \'{objective}\'')
-            print(f'Known objectives: {Constraint.get_weight_and_objectives()}')
+            print(f"Unknown dose objective: '{objective}'")
+            print(f"Known objectives: {Constraint.get_weight_and_objectives()}")
             return None
 
         # Return pre-existing result if available.
@@ -925,17 +1079,30 @@ class Plan(skrt.core.Archive):
             dose_objective = Dose(self.doses[idx_dose])
 
         dose_objective.load()
-        dose_objective.data = np.zeros(dose_objective.data.shape)
-        
+        dose_objective.data = (-1) * np.ones(dose_objective.data.shape)
+
         # Obtain objective information for each ROI.
         for roi in rois:
             if roi.constraint:
                 roi_clone = skrt.structures.ROI(roi)
                 roi_clone.set_image(dose_objective)
                 mask = roi_clone.get_mask()
-                dose_objective.data[mask > 0] = (
-                        getattr(roi_clone.constraint, objective))
+                value = getattr(roi_clone.constraint, objective)
+                if value is None:
+                    continue
+                if "maximum_dose" == objective:
+                    dose_objective.data[np.where(
+                        (mask > 0) & ((dose_objective.data < 0) |
+                                      (dose_objective.data > value)))] = value
+                elif "minimum_dose" == objective:
+                    dose_objective.data[np.where(
+                        (mask > 0) & ((dose_objective.data < 0) |
+                                      (dose_objective.data < value)))] = value
+                else:
+                    dose_objective.data[mask > 0] = value
 
+            dose_objective.data = np.ma.masked_where(
+                    dose_objective.data < 0, dose_objective.data)
         setattr(self.objectives, objective, dose_objective)
 
         return dose_objective
@@ -950,6 +1117,22 @@ class Plan(skrt.core.Archive):
         self.load()
         return self.description
 
+    def get_station_name(self):
+        """Return station name."""
+        self.load()
+        return self.station_name
+
+    def get_machine(self, machines=None):
+        """
+        Return machine name.
+
+        **Parameter:**
+
+        machines: dict, default=None
+            Dictionary mapping between station names and machine names.
+        """
+        return skrt.image.Image.get_machine(self, machines)
+
     def get_prescription_description(self):
         """Return plan prescription description."""
         self.load()
@@ -1003,7 +1186,8 @@ class Plan(skrt.core.Archive):
 
         # Convert to StructureSet object if needed
         if structure_set and not isinstance(
-                structure_set, skrt.structures.StructureSet):
+            structure_set, skrt.structures.StructureSet
+        ):
             structure_set = skrt.structures.StructureSet(structure_set)
 
         # Assign structure set to self
@@ -1039,21 +1223,28 @@ class Plan(skrt.core.Archive):
         """Return Dose objects associated with this plan."""
         return self.doses
 
+
 class Constraint(skrt.core.Data):
-    '''
+    """
     Container for data relating to a dose constraint.
-    '''
+    """
 
     @classmethod
     def get_weight_and_objectives(cls):
-        '''
+        """
         Return weight and objectives that may be associated with a constraint.
-        '''
-        return ['weight', 'minimum_dose', 'maximum_dose', 'full_volume_dose',
-                'prescription_dose', 'underdose_volume_fraction',
-                'overdose_volume_fraction']
+        """
+        return [
+            "weight",
+            "minimum_dose",
+            "maximum_dose",
+            "full_volume_dose",
+            "prescription_dose",
+            "underdose_volume_fraction",
+            "overdose_volume_fraction",
+        ]
 
-    def __init__(self, opts={}, **kwargs):
+    def __init__(self, opts=None, **kwargs):
         """
         Constructor of Container class.
 
@@ -1070,7 +1261,7 @@ class Constraint(skrt.core.Data):
 
         **Parameters:**
 
-        opts: dict, default={}
+        opts: dict, default=None
             Dictionary to be used in setting instance attributes
             (dictionary keys) and their initial values.
 
@@ -1082,10 +1273,11 @@ class Constraint(skrt.core.Data):
         for attribute in Constraint.get_weight_and_objectives():
             setattr(self, attribute, None)
 
-        super().__init__(opts, **kwargs)
+        super().__init__(opts or {}, **kwargs)
+
 
 def remove_duplicate_doses(doses=None):
-    '''
+    """
     Remove duplicates from a list of dose objects.
 
     Dose instance dose1 is taken to be a duplicate of dose2 if
@@ -1094,11 +1286,12 @@ def remove_duplicate_doses(doses=None):
     **Parameter:**
     doses: list, default=None
         List of dose objects, from which duplicates are to be removed.
-    '''
+    """
     return skrt.image.remove_duplicate_images(doses)
 
+
 def sum_doses(doses=None):
-    '''
+    """
     Sum doses of the same geometry.
 
     If not all doses have the same same geometry (shape, origin,
@@ -1110,5 +1303,5 @@ def sum_doses(doses=None):
     **Parameter:**
     doses: list, default=None
         List of Dose objects to be summed.
-    '''
+    """
     return skrt.image.sum_images(doses)
